@@ -18,7 +18,7 @@ use crate::CliResult;
 use dateparser::DateTimeUtc;
 use serde::Deserialize;
 
-use self::FieldType::{TDate, TFloat, TInteger, TNull, TString, TUnknown};
+use self::FieldType::{TDate, TDateTime, TFloat, TInteger, TNull, TString, TUnknown};
 
 static USAGE: &str = "
 Computes basic statistics on CSV data.
@@ -31,11 +31,11 @@ min/max length, mean, stddev & variance. The default set of statistics correspon
 statistics that can be computed efficiently on a stream of data (i.e., constant memory).
 
 The data type of each column is also inferred (Unknown, NULL, Integer, String,
-Float and Date). The date formats recognized can be found at
+Float, Date and DateTime). The date formats recognized can be found at
 https://docs.rs/dateparser/0.1.6/dateparser/#accepted-date-formats.
 
 Computing statistics on a large file can be made much faster if you create
-an index for it first with 'qsv index' as enables multithreading.
+an index for it first with 'qsv index' to enable multithreading.
 
 Usage:
     qsv stats [options] [<input>]
@@ -58,6 +58,9 @@ stats options:
                            This requires storing all CSV data in memory.
     --nulls                Include NULLs in the population size for computing
                            mean and standard deviation.
+    --dates                Infer date/datetime datatypes. This is a very expensive
+                           option and should only be used when you know there
+                           are date/datetime fields.
     -j, --jobs <arg>       The number of jobs to run in parallel.
                            This works only when the given CSV has an index.
                            Note that a file handle is opened for each job.
@@ -87,6 +90,7 @@ pub struct Args {
     pub flag_median: bool,
     pub flag_quartiles: bool,
     pub flag_nulls: bool,
+    pub flag_dates: bool,
     pub flag_nullcount: bool,
     pub flag_jobs: isize,
     pub flag_output: Option<String>,
@@ -161,7 +165,7 @@ impl Args {
             });
         }
         drop(send);
-        Ok((headers, merge_all(recv.iter()).unwrap_or_else(Vec::new)))
+        Ok((headers, merge_all(recv.iter()).unwrap_or_default()))
     }
 
     pub fn stats_to_records(&self, stats: Vec<Stats>) -> Vec<csv::StringRecord> {
@@ -238,6 +242,7 @@ impl Args {
             median: self.flag_median && !self.flag_quartiles && !self.flag_everything,
             quartiles: self.flag_quartiles || self.flag_everything,
             mode: self.flag_mode || self.flag_everything,
+            dates: self.flag_dates,
         }))
         .take(record_len)
         .collect()
@@ -293,9 +298,11 @@ struct WhichStats {
     median: bool,
     quartiles: bool,
     mode: bool,
+    dates: bool,
 }
 
 impl Commute for WhichStats {
+    #[inline]
     fn merge(&mut self, other: WhichStats) {
         assert_eq!(*self, other);
     }
@@ -352,7 +359,7 @@ impl Stats {
     #[allow(clippy::option_map_unit_fn)]
     #[inline]
     fn add(&mut self, sample: &[u8]) {
-        let sample_type = FieldType::from_sample(sample);
+        let sample_type = FieldType::from_sample(&self.which.dates, sample);
         self.typ.merge(sample_type);
 
         let t = self.typ;
@@ -398,14 +405,14 @@ impl Stats {
                     });
                 }
             }
-            TDate => {}
+            _ => {}
         }
     }
 
     #[allow(clippy::wrong_self_convention)]
     pub fn to_record(&mut self) -> csv::StringRecord {
         let typ = self.typ;
-        let mut pieces = vec![];
+        let mut pieces = Vec::with_capacity(20);
         let empty = || "".to_owned();
 
         pieces.push(self.typ.to_string());
@@ -524,6 +531,7 @@ impl Stats {
 }
 
 impl Commute for Stats {
+    #[inline]
     fn merge(&mut self, other: Stats) {
         self.typ.merge(other.typ);
         self.sum.merge(other.sum);
@@ -537,6 +545,7 @@ impl Commute for Stats {
     }
 }
 
+#[allow(clippy::enum_variant_names)]
 #[derive(Clone, Copy, PartialEq, Eq, Hash)]
 pub enum FieldType {
     TUnknown,
@@ -545,10 +554,12 @@ pub enum FieldType {
     TFloat,
     TInteger,
     TDate,
+    TDateTime,
 }
 
 impl FieldType {
-    pub fn from_sample(sample: &[u8]) -> FieldType {
+    #[inline]
+    pub fn from_sample(dates: &bool, sample: &[u8]) -> FieldType {
         if sample.is_empty() {
             return TNull;
         }
@@ -562,9 +573,19 @@ impl FieldType {
         if string.parse::<f64>().is_ok() {
             return TFloat;
         }
-        if string.parse::<DateTimeUtc>().is_ok() {
-            return TDate;
+        if *dates {
+            if let Ok(parsed_date) = string.parse::<DateTimeUtc>() {
+                let rfc3339_date_str = parsed_date.0.to_string();
+                let datelen = rfc3339_date_str.len();
+
+                if datelen >= 17 {
+                    return TDateTime;
+                } else {
+                    return TDate;
+                }
+            }
         }
+
         TString
     }
 
@@ -578,25 +599,32 @@ impl FieldType {
 }
 
 impl Commute for FieldType {
+    #[inline]
     fn merge(&mut self, other: FieldType) {
         *self = match (*self, other) {
             (TString, TString) => TString,
             (TFloat, TFloat) => TFloat,
             (TInteger, TInteger) => TInteger,
             (TDate, TDate) => TDate,
+            (TDateTime, TDateTime) => TDateTime,
             // Null does not impact the type.
             (TNull, any) | (any, TNull) => any,
             // There's no way to get around an unknown.
             (TUnknown, _) | (_, TUnknown) => TUnknown,
+            // date data types
+            (TDate, TDateTime) | (TDateTime, TDate) => TDateTime,
             // Integers can degrade to floats.
             (TFloat, TInteger) | (TInteger, TFloat) => TFloat,
             // when using unixtime format can degrade to int/floats.
             (TInteger, TDate) | (TDate, TInteger) => TInteger,
             (TFloat, TDate) | (TDate, TFloat) => TFloat,
+            (TInteger, TDateTime) | (TDateTime, TInteger) => TInteger,
+            (TFloat, TDateTime) | (TDateTime, TFloat) => TFloat,
             // Numbers/dates can degrade to unicode Strings.
             (TString, TFloat) | (TFloat, TString) => TString,
             (TString, TInteger) | (TInteger, TString) => TString,
             (TString, TDate) | (TDate, TString) => TString,
+            (TString, TDateTime) | (TDateTime, TString) => TString,
         };
     }
 }
@@ -619,6 +647,7 @@ impl fmt::Display for FieldType {
             TFloat => write!(f, "Float"),
             TInteger => write!(f, "Integer"),
             TDate => write!(f, "Date"),
+            TDateTime => write!(f, "DateTime"),
         }
     }
 }
@@ -632,6 +661,7 @@ impl fmt::Debug for FieldType {
             TFloat => write!(f, "Float"),
             TInteger => write!(f, "Integer"),
             TDate => write!(f, "Date"),
+            TDateTime => write!(f, "DateTime"),
         }
     }
 }
@@ -674,9 +704,10 @@ impl TypedSum {
         }
     }
 
+    #[inline]
     fn show(&self, typ: FieldType) -> Option<String> {
         match typ {
-            TNull | TString | TUnknown | TDate => None,
+            TNull | TString | TUnknown | TDate | TDateTime => None,
             TInteger => Some(self.integer.to_string()),
             TFloat => Some(self.float.unwrap_or(0.0).to_string()),
         }
@@ -684,6 +715,7 @@ impl TypedSum {
 }
 
 impl Commute for TypedSum {
+    #[inline]
     fn merge(&mut self, other: TypedSum) {
         match (self.float, other.float) {
             (Some(f1), Some(f2)) => self.float = Some(f1 + f2),
@@ -731,7 +763,7 @@ impl TypedMinMax {
                 self.integers.add(n);
                 self.floats.add(n as f64);
             }
-            TDate => {
+            TDate | TDateTime => {
                 let n = str::from_utf8(&*sample)
                     .ok()
                     .and_then(|s| dateparser::parse(s).ok())
@@ -742,6 +774,7 @@ impl TypedMinMax {
         }
     }
 
+    #[inline]
     fn len_range(&self) -> Option<(String, String)> {
         match (self.str_len.min(), self.str_len.max()) {
             (Some(min), Some(max)) => Some((min.to_string(), max.to_string())),
@@ -749,6 +782,7 @@ impl TypedMinMax {
         }
     }
 
+    #[inline]
     fn show(&self, typ: FieldType) -> Option<(String, String)> {
         match typ {
             TNull => None,
@@ -760,7 +794,7 @@ impl TypedMinMax {
                 }
                 _ => None,
             },
-            TDate => match (self.dates.min(), self.dates.max()) {
+            TDate | TDateTime => match (self.dates.min(), self.dates.max()) {
                 (Some(min), Some(max)) => Some((min.to_string(), max.to_string())),
                 _ => None,
             },
@@ -777,6 +811,7 @@ impl TypedMinMax {
 }
 
 impl Commute for TypedMinMax {
+    #[inline]
     fn merge(&mut self, other: TypedMinMax) {
         self.strings.merge(other.strings);
         self.str_len.merge(other.str_len);
